local assets = {
    Asset("ANIM", "anim/bflare.zip"),
    Asset("ANIM", "anim/swap_bflare.zip"),	
    Asset("ATLAS", "images/inventoryimages/bflare.xml"),
    Asset("IMAGE", "images/inventoryimages/bflare.tex"),
}

local prefabs =
{

	"fire_projectile",
	"cutgrass",
    
}

---------RED STAFF---------

local function onattack_red(inst, attacker, target, skipsanity)
    if not skipsanity and attacker ~= nil and attacker.components.sanity ~= nil then
        attacker.components.sanity:DoDelta(0)
    end

    attacker.SoundEmitter:PlaySound("dontstarve/wilson/fireball_explo")

    if not target:IsValid() then
        --target killed or removed in combat damage phase
        return
    elseif target.components.burnable ~= nil and not target.components.burnable:IsBurning() then
        if target.components.freezable ~= nil and target.components.freezable:IsFrozen() then
            target.components.freezable:Unfreeze()
        elseif target.components.fueled == nil
            or (target.components.fueled.fueltype ~= FUELTYPE.BURNABLE and
                target.components.fueled.secondaryfueltype ~= FUELTYPE.BURNABLE) then
            --does not take burnable fuel, so just burn it
            if target.components.burnable.canlight or target.components.combat ~= nil then
                target.components.burnable:Ignite(true)
            end
        elseif target.components.fueled.accepting then
            --takes burnable fuel, so fuel it
            local fuel = SpawnPrefab("cutgrass")
            if fuel ~= nil then
                if fuel.components.fuel ~= nil and
                    fuel.components.fuel.fueltype == FUELTYPE.BURNABLE then
                    target.components.fueled:TakeFuelItem(fuel)
                else
                    fuel:Remove()
                end
            end
        end
    end

    if target.components.freezable ~= nil then
        target.components.freezable:AddColdness(-1) --Does this break ice staff?
        if target.components.freezable:IsFrozen() then
            target.components.freezable:Unfreeze()
        end
    end

    if target.components.sleeper ~= nil and target.components.sleeper:IsAsleep() then
        target.components.sleeper:WakeUp()
    end

    if target.components.combat ~= nil then
        target.components.combat:SuggestTarget(attacker)
    end

    target:PushEvent("attacked", { attacker = attacker, damage = 0, weapon = inst })
end

local function onlight(inst, target)
    if inst.components.finiteuses ~= nil then
        inst.components.finiteuses:Use(1)
    end
end

local function onhauntred(inst, haunter)
    if math.random() <= TUNING.HAUNT_CHANCE_RARE then
        local x, y, z = inst.Transform:GetWorldPosition() 
        local ents = TheSim:FindEntities(x, y, z, 6, { "canlight" }, { "fire", "burnt", "INLIMBO" })
        if #ents > 0 then
            for i, v in ipairs(ents) do
                if v:IsValid() and not v:IsInLimbo() then
                    onattack_red(inst, haunter, v, true) 
                end
            end
            inst.components.hauntable.hauntvalue = TUNING.HAUNT_LARGE
            return true
        end
    end
    return false
end

---------COMMON FUNCTIONS---------

--local function onfinished(inst)
--    inst.SoundEmitter:PlaySound("dontstarve/common/gem_shatter")
--    inst:Remove()
--end

local function turnon(inst) -- enable light
    inst.Light:Enable(true)
end
 
local function turnoff(inst) --disable light
    inst.Light:Enable(false)
end

local function onunequip(inst, owner)
    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")
	turnoff(inst)
end

local function CreateLight()
    local inst = CreateEntity()

    inst:AddTag("FX")
    inst:AddTag("playerlight")
    --[[Non-networked entity]]
    inst.entity:SetCanSleep(false)
    inst.persists = false

    inst.entity:AddTransform()
    inst.entity:AddLight()

    inst.Light:SetIntensity(.75)
    inst.Light:SetColour(197 / 255, 197 / 255, 50 / 255)
    inst.Light:SetFalloff(.5)
    inst.Light:SetRadius(2)

    return inst
end

local function AttachLightTo(inst, target)
    inst._light.entity:SetParent(target.entity)
end


local function commonfn(colour, tags)
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddNetwork()
	inst.entity:AddLight()

    MakeInventoryPhysics(inst)

    inst.AnimState:SetBank("bflare")
    inst.AnimState:SetBuild("bflare")
    inst.AnimState:PlayAnimation("idle")
	
	inst.Light:SetIntensity(.75)
    inst.Light:SetColour(197 / 255, 197 / 255, 50 / 255)
    inst.Light:SetFalloff(.5)
    inst.Light:SetRadius(2)
	


    if tags ~= nil then
        for i, v in ipairs(tags) do
            inst:AddTag(v)
        end
    end

    local floater_swap_data =
    {
        sym_build = "swap_staffs",
        sym_name = "swap_"..colour.."staff",
        bank = "staffs",
        anim = colour.."staff"
    }
    MakeInventoryFloatable(inst, "med", 0.1, {0.9, 0.4, 0.9}, true, -13, floater_swap_data)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    -------
    --inst:AddComponent("finiteuses")
    --inst.components.finiteuses:SetOnFinished(onfinished)
	

    inst:AddComponent("inspectable")
	

    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.imagename = "bflare"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/bflare.xml"

    inst:AddComponent("tradable")

    inst:AddComponent("equippable")
	
	inst.components.equippable:SetOnUnequip(onunequip)
	
    inst.components.equippable:SetOnEquip(function(inst, owner)
        owner.AnimState:OverrideSymbol("swap_object", "swap_bflare", "swap_bflare")
        owner.AnimState:Show("ARM_carry")
        owner.AnimState:Hide("ARM_normal")
		turnon(inst)
    end)
    return inst
end

---------COLOUR SPECIFIC CONSTRUCTIONS---------

local function red()
    --weapon (from weapon component) added to pristine state for optimization
    local inst = commonfn("red", { "firestaff", "weapon", "rangedweapon", "rangedlighter" })

    inst.projectiledelay = FRAMES

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(50)
    inst.components.weapon:SetRange(8, 10)
    inst.components.weapon:SetOnAttack(onattack_red)
    inst.components.weapon:SetProjectile("fire_projectile")

    --inst.components.finiteuses:SetMaxUses(TUNING.FIRESTAFF_USES)
    --inst.components.finiteuses:SetUses(TUNING.FIRESTAFF_USES)

    local floater_swap_data =
    {
        sym_build = "swap_staffs",
        sym_name = "swap_redstaff",
        bank = "staffs",
        anim = "redstaff"
    }
    inst.components.floater:SetBankSwapOnFloat(true, -9.5, floater_swap_data)
    inst.components.floater:SetScale({0.85, 0.4, 0.85})

    MakeHauntableLaunch(inst)
    AddHauntableCustomReaction(inst, onhauntred, true, false, true)

    return inst
end

STRINGS.NAMES.BFLARE = "Burnish flare"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.BFLARE = "The given power of the burnish."

return Prefab("bflare", red, assets, prefabs)