local MakePlayerCharacter = require "prefabs/player_common"


local assets = {
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
    Asset("SOUNDPACKAGE", "sound/esc.fev"),
    Asset("SOUND", "sound/esc.fsb"),
}
local prefabs = {}

-- Custom starting inventory
local start_inv = {
	"bflare",
}

-- When the character is revived from human
local function onbecamehuman(inst)
	-- Set speed when not a ghost (optional)
	inst.components.locomotor:SetExternalSpeedMultiplier(inst, "koli_speed_mod", 1)
end

local function onbecameghost(inst)
	-- Remove speed modifier when becoming a ghost
   inst.components.locomotor:RemoveExternalSpeedMultiplier(inst, "koli_speed_mod")
end

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end


-- This initializes for both the server and client. Tags can be added here.
local common_postinit = function(inst) 
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "koli.tex" )
end

--神奇代码 火回san

local function sanityfn(inst)
	local x,y,z = inst.Transform:GetWorldPosition()	
	local delta = 0
	local max_rad = 10
	local ents = TheSim:FindEntities(x,y,z, max_rad, {"fire"})
    for k,v in pairs(ents) do
    	if v.components.burnable and v.components.burnable.burning then
    		local sz = TUNING.SANITYAURA_TINY
    		local rad = v.components.burnable:GetLargestLightRadius() or 1
    		sz = sz * ( math.min(max_rad, rad) / max_rad )
			local distsq = inst:GetDistanceSqToInst(v)
			delta = delta + sz/math.max(1, distsq)
    	end
    end
	
	return delta
end


-- This initializes for the server only. Components are added here.
local master_postinit = function(inst)
	-- choose which sounds this character will play
	inst.soundsname = "wilson"
	--神奇代码 火免
	inst.components.health.fire_damage_scale = 0
	-- Uncomment if "wathgrithr"(Wigfrid) or "webber" voice is used
    --inst.talker_path_override = "dontstarve_DLC001/characters/"
	
	-- custom sounds example, Work-In-Progress -M
	--inst.soundsname = "koli"
    --inst.talker_path_override = "esc/"
	
	-- Stats	
	inst.components.health:SetMaxHealth(150)
	inst.components.hunger:SetMax(150)
	inst.components.sanity:SetMax(200)
	
	-- Damage multiplier (optional)
    inst.components.combat.damagemultiplier = 1
	--test 过热防护
	inst.components.temperature.maxtemp = 60
	-- Hunger rate (optional)
	inst.components.hunger.hungerrate = 1 * TUNING.WILSON_HUNGER_RATE
	--神奇代码 火回san
	inst.components.sanity.custom_rate_fn = sanityfn
	
	inst.OnLoad = onload
    inst.OnNewSpawn = onload
	
end


local function OnExplode(inst, target)
    inst.AnimState:PlayAnimation("trap")
	
	--Burn the target
	
	if target.components.health then 
		target.components.health:DoFireDamage(TUNING.FIRETRAP_DAMAGE, inst, true)
	end
	
	if target.components.burnable then
		target.components.burnable:Ignite(true, inst)
	end
	
	--Thow fire everywhere!
	if inst.components.lootdropper == nil then
		inst:AddComponent("lootdropper")
	end
	for i = 1, TUNING.FIRETRAP_FIRECOUNT do
		inst.components.lootdropper:SpawnLootPrefab("firetrap_fire", inst:GetPosition())
	end	
	
    --[[
	if target then
        inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_trigger")
        target.components.combat:GetAttacked(inst, TUNING.TRAP_TEETH_DAMAGE)
    end
	]]
    if inst.components.finiteuses then
        inst.components.finiteuses:Use(1)
    end
end

return MakePlayerCharacter("koli", prefabs, assets, common_postinit, master_postinit, start_inv)
